/*
 * Copyright (c) 1998-99 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */

package org.apache.jmeter.samplers;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.IOException;
import java.net.URL;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.swing.*;

/**
 * This class provides a way to provide Authorization in jmeter requests.
 * The format of the authorization file is:
 *
 * URL	user	pass
 *
 * where URL is an HTTP URL, user a username to use and pass the appropriate
 * password.
 *
 * @author  <a href="mailto:luta.raphael@networks.vivendi.com">Raphal Luta</a>
 * @version 0.9
 */
public class AuthManager extends AbstractTableModel implements Cloneable {
    /**
     * A vector of Authorization objects managed by this class.
     * @associates <{org.apache.jmeter.samplers.Authorization}>
     * @supplierCardinality 0..**/
    private Vector auths = new Vector();

    private final static int columnCount = 3;
    private final static String[] columnNames = {"Base URL","Username","Password"};
    /** Default Constructor */
    public AuthManager () {
    }
    /** required by table model interface */
   public int getRowCount() {
       return auths.size();
   }
   /** required by table model interface */
   public int getColumnCount() {
       return columnCount;
   }
   /** required by table model interface */
   public String getColumnName(int column) {
       return columnNames[column];
   }
   /** required by table model interface */
   public Class getColumnClass(int column) {
       return columnNames[column].getClass();
   }
   /** required by table model interface */
   public Object getValueAt(int row, int column) {
	Authorization auth = (Authorization) auths.elementAt(row);
        if (column == 0) {
            return auth.getURL();
        } else if (column == 1) {
            return auth.getUser();
        } else if (column == 2) {
            return auth.getPass();
        }
	return null;
   }
    /** save the authentication data to a file */
   public void save(String authFile) throws IOException {
       File file = new File(authFile);
        if (!file.isAbsolute()) file = new File(System.getProperty("user.dir") + File.separator + authFile);
        PrintWriter writer = new PrintWriter(new FileWriter(file));
	writer.println("# JMeter generated Authorization file");
	for (int i = 0; i < auths.size(); i++) {
		Authorization auth = (Authorization) auths.elementAt(i);
		writer.println(auth.toString());
	}
	writer.flush();
	writer.close();
    }
    /** add authentication data from a file */
   public void addFile(String authFile) throws IOException {
        File file = new File(authFile);
        if (!file.isAbsolute()) file = new File(System.getProperty("user.dir") + File.separator + authFile);
        BufferedReader reader = null;
	if( file.canRead() ) {
		reader = new BufferedReader(new FileReader(file));
	} else {
		JOptionPane.showMessageDialog(null,"The file you specified cannot be read.",
					"Information",JOptionPane.INFORMATION_MESSAGE);
		return;
	}

        String line;
        while ((line = reader.readLine()) != null) {
            try {
                if (line.startsWith("#") || line.trim().length() == 0) continue;
                StringTokenizer st = new StringTokenizer(line, "\t");
                String url = st.nextToken();
                String user = st.nextToken();
                String pass = st.nextToken();
                Authorization auth = new Authorization(url, user, pass);
                auths.addElement(auth);
            } catch (Exception e) {
                throw new IOException("Error parsing auth line\n\t'" + line + "'\n\t" + e);
            }
        }
        reader.close();
        fireTableChanged(new TableModelEvent(this));
    }

   /** update an authentication record */
   public void set(int index, String url, String user, String pass) {
        Authorization auth = new Authorization(url, user, pass);
       if (index >= 0) {
           auths.setElementAt(auth, index);
       } else {
           auths.addElement(auth);
       }
       fireTableChanged(new TableModelEvent(this));
    }

   /** remove an authentication record */
   public void remove(int index) {
        auths.removeElementAt(index);
       fireTableChanged(new TableModelEvent(this));
    }
    
   /** return the number of records */
   public int size() {
       return auths.size();
   }
   
   /** return the record at index i */
   public Authorization get(int i) {
       return (Authorization) auths.elementAt(i);
   }

   public String getAuthHeaderForURL(URL url) {
        if (!url.getProtocol().toUpperCase().equals("HTTP")) return null;

        StringBuffer header = new StringBuffer();
        for (Enumeration enum = auths.elements(); enum.hasMoreElements();) {
            Authorization auth = (Authorization) enum.nextElement();
            if (url.toString().startsWith(auth.getURL())) {
                header.append("Basic "+Base64Encoder.encode(auth.getUser()+":"+auth.getPass()));
				break;
            }
        }

        if (header.length() != 0) {
            return header.toString();
        } else {
            return null;
        }
    }
}

/**
  * This class provides an implementation of Base64 encoding without
  * relying on the the sun.* packages.
  *
  */

class Base64Encoder {
  private static final char[] pem_array = { 65,66,67,68,69,70,71,72,73,74,
                                            75,76,77,78,79,80,81,82,83,84,
                                            85,86,87,88,89,90,97,98,99,100,
                                            101,102,103,104,105,106,107,108,
                                            109,110,111,112,113,114,115,116,
                                            117,118,119,120,121,122,48,49,50,
                                            51,52,53,54,55,56,57,43,47 };
  private static final char eq = 61;

  static { }

  public Base64Encoder() { }

  public static final String encode(String s) {
    return encode(s.getBytes());
  }

  public static final String encode(byte[] bs) {
    StringBuffer out = new StringBuffer("");
    int bl = bs.length;
    for(int i=0; i<bl; i+=3)
      out.append(encodeAtom(bs, i, (bl-i)));
    return out.toString();
  }

  public static final String encodeAtom(byte[] b, int strt, int left) {
    StringBuffer out = new StringBuffer("");
    if(left == 1) {
      byte b1 = b[strt];
      int k = 0;
      out.append(String.valueOf(pem_array[b1 >>> 2 & 63]));
      out.append(String.valueOf(pem_array[(b1 << 4 & 48) + (k >>> 4 & 15)]));
      out.append(String.valueOf(eq));
      out.append(String.valueOf(eq));
      return out.toString();
    }
    if(left == 2) {
      byte b2 = b[strt];
      byte b4 = b[strt + 1];
      int l = 0;
      out.append(String.valueOf(pem_array[b2 >>> 2 & 63]));
      out.append(String.valueOf(pem_array[(b2 << 4 & 48) + (b4 >>> 4 & 15)]));
      out.append(String.valueOf(pem_array[(b4 << 2 & 60) + (l >>> 6 & 3)]));
      out.append(String.valueOf(eq));
      return out.toString();
    }
    byte b3 = b[strt];
    byte b5 = b[strt + 1];
    byte b6 = b[strt + 2];
    out.append(String.valueOf(pem_array[b3 >>> 2 & 63]));
    out.append(String.valueOf(pem_array[(b3 << 4 & 48) + (b5 >>> 4 & 15)]));
    out.append(String.valueOf(pem_array[(b5 << 2 & 60) + (b6 >>> 6 & 3)]));
    out.append(String.valueOf(pem_array[b6 & 63]));
    return out.toString();
  }
}

