/*
 * Copyright (c) 1998-99 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */
package org.apache.jmeter.samplers;

import java.io.*;
import java.net.*;
import java.security.Security;

import org.apache.jmeter.JMeterUtils;

/**
 * A sampler which understands all the parts necessary to read
 * statistics about HTTP requests, including cookies and authentication.
 */
public class HTTPSampler implements Sampler {
    CookieManager cookieManager;
    AuthManager authManager;

    public HTTPSampler(CookieManager cm, AuthManager am) {
	this.cookieManager=cm;
	this.authManager=am;
    }

    public SampleResult sample(Entry e)
    {
      return sample(e,false);
    }

  private SampleResult sample(Entry e,boolean redirected)
  {
  	long time;
  	HttpURLConnection conn;
  	BufferedInputStream in;
  	byte[] buffer = new byte[4096];
  	SampleResult res = new SampleResult();
    URL u = null;
  	try {
      //make URL from Entry
	    u = constructURL(e);
	    res.putValue(Sampler.SAMPLE_LABEL,u.toString());
	    conn = (HttpURLConnection)u.openConnection();
      conn.setFollowRedirects(false);
	    if (((Boolean)e.getValue(POST)).booleanValue())
		    conn.setRequestMethod("POST");
	    if (cookieManager != null)
      {
    		String cookieHeader=cookieManager.getCookieHeaderForURL(u);
    		if (cookieHeader != null)
    	    conn.setRequestProperty("Cookie", cookieHeader);
	    }
	    if (authManager != null)
      {
    		String authHeader = authManager.getAuthHeaderForURL(u);
    		if (authHeader != null)
    	    conn.setRequestProperty("Authorization", authHeader);
	    }
      // if POSTing data, write data to output stream
      if (!redirected && ((Boolean)e.getValue(POST)).booleanValue())
        sendPostData(conn,e,(String)e.getValue(this.CONTENT_TYPE));
	    time = System.currentTimeMillis();
      conn.connect();
	    if (cookieManager!=null)
      {
        if (conn.getHeaderField("Set-Cookie")!=null)
        {
          for (int i = 1; conn.getHeaderFieldKey(i) != null; i++)
            if (conn.getHeaderFieldKey(i).equals ("Set-Cookie"))
            {
              cookieManager.addCookieFromHeader(conn.getHeaderField(i), u);
            }
		    }
	    }
      // Get the response status
      int errorLevel = 2;
      try {
        errorLevel = ((HttpURLConnection) conn).getResponseCode() / 100;
      }catch(Exception e2) {
          res.putValue(this.TEXT_RESPONSE,e2.toString());
          res.setTime(System.currentTimeMillis() - time);
          res.putValue(SUCCESS,new Boolean(false));
      }
      if (errorLevel == 2)
      {
        // Read the served response
        in = new BufferedInputStream(conn.getInputStream());
        java.io.ByteArrayOutputStream w = new ByteArrayOutputStream();
        int x = 0;
  	    while((x = in.read(buffer))!=-1)
          w.write(buffer,0,x);
  	    time = System.currentTimeMillis()-time;
  	    in.close();
        res.putValue(this.TEXT_RESPONSE,w.toString());
        res.putValue(SUCCESS,new Boolean(true));
      }
      else if (errorLevel == 3)
      {
        // Request was relocated
        String loc = conn.getHeaderField("Location");
        System.out.println("Original location="+loc);
        if (loc != null)
        {
          if (loc.indexOf("http") == -1)
          {
            String tempURL = u.toString();
            if(loc.startsWith("/"))
            {
              int ind = tempURL.indexOf("//")+2;
              loc = tempURL.substring(0,tempURL.indexOf("/",ind)+1) + loc.substring(1);
            }
            else
            {
              loc = u.toString().substring(0,
                    u.toString().lastIndexOf('/') + 1) + loc;
            }
          }
        }
        System.out.println("Modified location="+loc);
        e.putValue(HTTPSampler.URL,loc);
        time = System.currentTimeMillis()-time;
        res = sample(e,true);
        time += res.getTime();
      }
      else // Could not sample the URL
        throw new IOException(((HttpURLConnection) conn).getResponseMessage());
	    res.setTime(time);
	    return res;
	  }catch (IOException ex) {ex.printStackTrace();res.setTime((long)0);
            res.putValue(this.TEXT_RESPONSE,ex.toString());
            res.putValue(SUCCESS,new Boolean(false));
      }
	  return res;
  }

    private String getPostData(String[][] args)
    {
      StringBuffer buf = new StringBuffer();
      for(int x = 0;x < args.length;x++)
      {
        buf.append(args[x][0]+"="+args[x][1]);
        if(x+1 < args.length)
          buf.append("&");
      }
      return buf.toString();
    }

  protected URL constructURL(Entry e) throws MalformedURLException
  {
  	StringBuffer url = new StringBuffer((String)e.getValue(URL));
  	String[][] args = (String[][])e.getValue(ARGUMENTS);

  	if (args.length>0 && ((Boolean)e.getValue(GET)).booleanValue())
    {
	    url.append("?"+args[0][0]+"="+args[0][1]);
	    for(int i=1;i<args.length;i++)
		  url.append("&"+args[i][0]+"="+args[i][1]);
	  }
	  return new URL(url.toString());
  }

  /**
   * Writes form data in multipart format.
   */
  private void writeFormMultipartStyle(PrintWriter out,String name,String value)
  {
    out.print(CRLF);
    out.print("Content-disposition: form-data; name=\""+name+"\"");
    out.print(CRLF);
    out.print(CRLF);
    out.print(value);
    out.print(CRLF);
    out.println("--"+BOUNDARY);
  }

  /**
     * Send POST data from Entry to the open connection.
     *
     */
  public void sendPostData(URLConnection connection, Entry e,String
          contentType)
          throws IOException
  {
		((HttpURLConnection) connection).setRequestMethod("POST");
    if(contentType == null || contentType.equals(NORMAL_FORM))
    {
      String postData = getPostData((String[][])e.getValue(this.ARGUMENTS));
  		connection.setRequestProperty("Content-length", "" + postData.length());
  		connection.setRequestProperty("Content-type", "application/x-www-form-urlencoded");
  		connection.setDoOutput(true);

  		PrintWriter out = new PrintWriter(connection.getOutputStream());
  		out.print(postData);
  		out.close();
    }
    else if(contentType.equals(MULTIPART_FORM))
    {
      connection.setRequestProperty("Content-type","multipart/form-data, "+BOUNDARY);
      connection.setDoOutput(true);
      OutputStream o = connection.getOutputStream();
      PrintWriter out = new PrintWriter(o);
      out.print(CRLF);
      out.print(CRLF);
      out.print("--"+BOUNDARY);
      String[][] args = (String[][])e.getValue(this.ARGUMENTS);
      for(int i = 0;i < args.length;i++)
        writeFormMultipartStyle(out,args[i][0],args[i][1]);
      writeFileToURL(o,(String)e.getValue(this.FILE_NAME),
              (String)e.getValue(FILE_FIELD),(InputStream)e.getValue(FILE_DATA),
              (String)e.getValue(FILE_MIMETYPE));
      out.print("--");
      o.close();
    }
  }

  /**
   * Writes out the contents of a file in correct multipart format.
   */
  private void writeFileToURL(OutputStream o,String filename,String fieldname,
          InputStream in,String mimetype) throws IOException
  {
    PrintWriter w = new PrintWriter(o);
    BufferedOutputStream out = new BufferedOutputStream(o);
    w.print(CRLF);
    w.print("Content-disposition: form-data; name=\""+fieldname+"\"; filename=\""+
            filename+"\"");
    w.print(CRLF);
    w.print("Content-Type: "+mimetype);
    w.print(CRLF);
    w.print(CRLF);

    byte[] buf = new byte[1024*100]; //100k
    int read;
    while((read = in.read(buf)) != 0)
      out.write(buf,0,read);
    w.print(CRLF);
    w.print("--"+BOUNDARY);
  }

    public static final String ARGUMENTS = "httpsampler.Arguments";
    public static final String URL = "httpsampler.URL";
    public static final String POST = "httpsampler.POST";
    public static final String GET = "httpsampler.GET";
    public static final String FILE_NAME = "httpsampler.FILE_NAME";
    public static final String FILE_FIELD = "httpsampler.FILE_FIELD";
    public static final String FILE_DATA = "httpsampler.FILE_DATA";
    public static final String FILE_MIMETYPE = "httpsampler.FILE_MIMETYPE";
    public static final String CONTENT_TYPE = "httpsampler.CONTENT_TYPE";
    public static final String NORMAL_FORM = "normal_form";
    public static final String MULTIPART_FORM = "multipart_form";

    private static final String BOUNDARY = "----------------BbAaJj{5002}Zz";
    private static final String CRLF = "\r\n";

}
