/*
 * Copyright (c) 1998-99 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */

package org.apache.jmeter.samplers.jdbc;
import java.sql.*;
import java.util.*;

/*****************************************************************
 This class manages a pool of Connection objects (ConnectionObject).  This
 pool is constantly checked for old, over-used, or dead connections in a
 separated thread.  Connections are rented out and then given back by the
 DBConnect object and its subclasses.  This class is not directly accessed
 by the end-user objects.  It is accessed by the DBConnect object and its
 subclasses.
@author Michael Stover
@version 1.0 10/13/1998
 *****************************************************************/
public class DBConnectionManager
{
  static int absoluteMaxConnections=100;
  static long accessInterval=1800000;
  static Hashtable connections;
  static Hashtable rentedConnections;

/******************************************************************
Constructor.
*****************************************************************/
  public DBConnectionManager()
  {
    if(connections==null)
      connections=new Hashtable();
    if(rentedConnections == null)
      rentedConnections=new Hashtable();
  }

/*********************************************************************
  Starts the connection manager going for a given database connection, and
  returns the DBKey object required to get a Connection object for this
  database.
@param url URL of database to be connected to.
@param username Username to use to connect to database.
@param password Password to use to connect to database.
@param driver Driver to use for the database.
@param maxUsage Sets the maxUsage parameter for connections to this database.
@param maxConnections Tells the DBConnectionManager how many connections to keep active.
@return DBKey object. Returns null if connection fails.
**********************************************************************/
  public DBKey startDatabaseConnection(String url,String username,String password,
          String driver,int maxUsage,int maxConnections)
  {
    DBKey key=new DBKey();
    if(registerDriver(driver))
    {
      key.setDriver(driver);
      key.setMaxConnections(maxConnections);
      key.setMaxUsage(maxUsage);
      key.setPassword(password);
      key.setUrl(url);
      key.setUsername(username);
      if(!connections.containsKey(key))
        setup(key);
    }
    else
      key=null;
    return key;
  }

  /******************************************************
 Constructor.
@param key DBKey that holds all information needed to set up a set of
        connections.
  ******************************************************/
  public void setup(DBKey key)
  {
    /* Code used to create a JDBC log for debuggin purposes
    try{
    java.io.PrintWriter jdbcLog=new java.io.PrintWriter(new java.io.FileWriter(jdbcLogFile));
    DriverManager.setLogWriter(jdbcLog);
    }catch(Exception e){}*/
    String url=key.getUrl();
    String username=key.getUsername();
    String password=key.getPassword();
    int maxConnections=key.getMaxConnections();
    int maxUsage=key.getMaxUsage();
    ConnectionObject[] connectionArray;
    int dbMax;
    try{
      DriverManager.registerDriver((java.sql.Driver)Class.forName(key.getDriver()).newInstance());
      DatabaseMetaData md=DriverManager.getConnection(url,username,password).getMetaData();
      dbMax=md.getMaxConnections();
      if(dbMax>0 && maxConnections>dbMax)
      {
        maxConnections=dbMax;
        key.setMaxConnections(maxConnections);
      }
      else if(maxConnections>absoluteMaxConnections)
      {
        maxConnections=absoluteMaxConnections;
        key.setMaxConnections(maxConnections);
      }
    }catch(Exception e){e.printStackTrace();
      maxConnections=1;}
    connectionArray=new ConnectionObject[maxConnections];
    int count=-1;
    while(++count<maxConnections)
      connectionArray[count]=new ConnectionObject(this,key);
    connections.put(key,connectionArray);
    System.gc();
  } // End Method




  /******************************************************
  Rents out a database connection object.
@return Connection object.
  ******************************************************/
  public Connection getConnection(DBKey key)      //deleted synchronized
  {
    ConnectionObject[] connectionArray=(ConnectionObject[])connections.get(key);
    int maxConnections=key.getMaxConnections();
    Connection c=null;
    int index=(int)(100*Math.random());
    int count=-1;
    while(++count<maxConnections && c==null)
    {
      index++;
      c=connectionArray[index%maxConnections].grab();
    }
    if(c!=null)
      rentedConnections.put(c,connectionArray[index%maxConnections]);
    return c;
  } // End Method

  /******************************************************
  Releases a connection back to the pool
@param c Connection object being returned
  ******************************************************/
  public void releaseConnection(Connection c)         // deleted synchronized
  {
    ConnectionObject connOb=(ConnectionObject)rentedConnections.get(c);
    if(connOb!=null)
    {
      connOb.release();
      rentedConnections.remove(c);
    }
    else
    {
      System.out.println("DBConnectionManager: Lost a connection connection='"+c+"'\r\n");
      c=null;
    }
  } // End Method


 /*********************************************************
  Returns a new java.sql.Connection object.
@throws java.sql.SQLException
 *********************************************************/
  public Connection newConnection(DBKey key) throws SQLException       //deleted synchronized
  {
    Connection c;
    c=DriverManager.getConnection(key.getUrl(),key.getUsername(),key.getPassword());
    return c;
  }

/*************************************************************
Closes out this object and returns resources to the system.
*************************************************************/
  public void close(DBKey key)
  {
    ConnectionObject[] connectionArray=(ConnectionObject[])connections.get(key);
    int count=-1;
    while(++count<connectionArray.length)
    {
      connectionArray[count].close();
      connectionArray[count]=null;
    }
    connections.remove(key);
  }



/*********************************************************************
Registers a driver for a database.
@param driver full classname for the driver.
@return True if successful, false otherwise.
***********************************************************************/
  public boolean registerDriver(String driver)
  {
    try{
      DriverManager.registerDriver((Driver)Class.forName(driver).newInstance());
    }catch(Exception e){e.printStackTrace(); return false;}
    return true;
  }


/*******************************************************************
 Private method to check if database exists.
@return True if database exists, false otherwise
 ******************************************************************
  private synchronized boolean checkForDatabase()
  {
    boolean connected=true;
    try
    {
      DatabaseMetaData dmd=connection[counter].getCon().getMetaData();
      int cons=dmd.getMaxConnections();
    }catch(SQLException e){connected=false;e.printStackTrace();}

    return connected;
  }  //end of method      */
}





