/*
 * Copyright (c) 1998-99 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */
package org.apache.jmeter.threads;

import java.util.*;
import org.apache.jmeter.timers.Timer;
import org.apache.jmeter.samplers.*;
import org.apache.jmeter.controllers.*;

/**
 * Provides some basic functionality required for successfully
 * implementing JMeterThread, to allow the subclasses to
 * focus on their logic rather than housekeeping.
 */
public class JMeterThreadAdapter implements JMeterThread
{
    /**
     * @associates <{org.apache.jmeter.threads.SampleListener}>
     */
    protected Vector listeners = new Vector();
    protected String threadGroup;
    protected boolean run = true;
    protected Sampler sampler;
    protected TestSample[] samples;
    protected Timer timer;


    public JMeterThreadAdapter(Sampler sampler, TestSample[] entries)
    {
	this.sampler=sampler;
	this.samples=entries;
        for(int x = 0;x < entries.length;x++)
          entries[x].reset();
    }

    public JMeterThreadAdapter(Sampler sampler, TestSample[] entries, String t)
    {
	this(sampler, entries);
	threadGroup = t;
    }

    public JMeterThreadAdapter(Sampler sampler, TestSample[] entries,
			String t,Vector listeners)
    {
	this(sampler, entries, t);
	this.listeners=listeners;
    }

    public void stopSampling()
    {
      run = false;
    }

    public void run() {
	int index=0;
	SampleResult res;
        Entry[] entries = getEntries();
	timer.set();
	while(run && entries.length > 0) {
	    timer.delay();
	    fireSampleStartedEvent(threadGroup);
	    res = sampler.sample(entries[index]);
	    fireSampleStoppedEvent(threadGroup);
	    fireSampleOccurredEvent(res, threadGroup);
	    index=(index+1)%entries.length;
            Thread.currentThread().yield();
	}
    }

    protected Entry[] getEntries() {
	Entry[] entries=null;
	Vector v = new Vector();

	if (samples!=null) {
	    for(int i=0;i<samples.length;i++) {
		entries = samples[i].getEntries();
		for(int j=0;j<entries.length;j++)
		    v.addElement(entries[j]);
	    }
	    entries = new Entry[v.size()];
	    v.copyInto(entries);

	    return entries;
	}
	return new Entry[0];
    }

    protected SampleResult sample(Entry e, String threadGroup) {
      try{
	Thread.currentThread().sleep(timer.delay());
      }catch(InterruptedException err){}
	fireSampleStartedEvent(threadGroup);
	SampleResult res = sampler.sample(e);
	fireSampleStoppedEvent(threadGroup);
  if(res != null)
	  fireSampleOccurredEvent(res, threadGroup);

	return res;
    }

    public String getThreadGroupName() {
	return threadGroup;
    }

    public void addSampleListener(SampleListener l)
    {
	synchronized(listeners) {
	    listeners.addElement(l);
	}
    }

    public void setTimer(Timer t) {
	timer=t;
    }

    public void removeSampleListener(SampleListener l)
    {
	synchronized(listeners) {
	    listeners.removeElement(l);
	}
    }

    protected void fireSampleOccurredEvent(SampleResult result,
					   String threadGroup)
    {
	SampleEvent e = new SampleEvent(result,threadGroup);
	synchronized(listeners) {
	    for(int i=0;i<listeners.size();i++) {
		SampleListener l = (SampleListener)listeners.elementAt(i);
		l.sampleOccurred(e);
	    }
	}
    }

    protected void fireSampleStartedEvent(String threadGroup)
    {
      synchronized(listeners)
      {
	  for(int i=0;i<listeners.size();i++) {
	      SampleListener l = (SampleListener)listeners.elementAt(i);
	      l.sampleStarted(new SampleEvent(null,threadGroup));
	  }
      }
    }

    protected void fireSampleStoppedEvent(String threadGroup)
    {
	synchronized(listeners) {
	    for(int i=0;i<listeners.size();i++) {
		SampleListener l = (SampleListener)listeners.elementAt(i);
		l.sampleStopped(new SampleEvent(null,threadGroup));
	    }
	}
    }
}
