/*
 * Copyright (c) 1998-99 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */

package org.apache.jmeter.visualizers;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import org.apache.jmeter.JMeterUtils;
import org.apache.jmeter.samplers.SampleResult;
import org.apache.jmeter.samplers.Sampler;
/**
 * This class implements a visualizer that writes the samples to a file.
 *
 * @author  <a href="mailto:stefano@apache.org">Stefano Mazzocchi</a>
 * @version $Revision: 1.11 $ $Date: 2000/09/06 01:18:33 $
 */
public class FileVisualizer extends JPanel implements Visualizer, ActionListener {

    private boolean verbose = true;
    private boolean append = true;
    private boolean open = false;
    private String file;
    private PrintWriter stream;

    JButton browseButton, openButton, flushButton, closeButton;
    JFileChooser fileChooser;
    JCheckBox appendCheck, verboseCheck;
    JComboBox fileList;

    public FileVisualizer() {
        super();

        // File panel
        JPanel filePanel = new JPanel();
        filePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Output File"));
        GridBagLayout g = new GridBagLayout();
        filePanel.setLayout(g);
        GridBagConstraints c = new GridBagConstraints();
        c.fill = c.BOTH;

        fileList = new JComboBox();
        fileList.setEditable(true);
        fileList.addActionListener(this);
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = 4;
        c.gridheight = 1;
        c.weightx = 1.0;
        c.weighty = 0.0;
        c.insets = new Insets(3, 3, 3, 3);
        g.setConstraints(fileList, c);
        filePanel.add(fileList);

        browseButton = new JButton("Browse");
        browseButton.addActionListener(this);
        browseButton.setEnabled(true);
        c.gridwidth = 1;
        c.gridx = 4;
        g.setConstraints(browseButton, c);
        filePanel.add(browseButton);

        appendCheck = new JCheckBox("Append", append);
        appendCheck.addActionListener(this);
        c.gridx = 0;
        c.gridy = 1;
        g.setConstraints(appendCheck, c);
        filePanel.add(appendCheck);

        verboseCheck = new JCheckBox("Verbose Output", verbose);
        verboseCheck.addActionListener(this);
        c.gridx = 1;
        g.setConstraints(verboseCheck, c);
        filePanel.add(verboseCheck);

        openButton = new JButton("Open");
        openButton.addActionListener(this);
        openButton.setEnabled(true);
        c.gridx = 2;
        g.setConstraints(openButton, c);
        filePanel.add(openButton);

        flushButton = new JButton("Flush");
        flushButton.addActionListener(this);
        flushButton.setEnabled(false);
        c.gridx = 3;
        g.setConstraints(flushButton, c);
        filePanel.add(flushButton);

        closeButton = new JButton("Close");
        closeButton.addActionListener(this);
        closeButton.setEnabled(false);
        c.gridx = 4;
        g.setConstraints(closeButton, c);
        filePanel.add(closeButton);

        fileChooser = new JFileChooser();

        this.add(filePanel);
    }

    public void setFile(String file) {
	this.file = file;
	fileList.removeItem(file);
	fileList.addItem(file);
	fileList.getEditor().setItem(file);
    }

    public void open() throws IOException {
	this.stream = new PrintWriter(new BufferedOutputStream(new FileOutputStream(this.file, this.append)), false);
	if (verbose) {
	    this.stream.println("\n# Sample data created by " + JMeterUtils.getJMeterProperties().getProperty("version"));
	    this.stream.println("\n#       URL                    Milliseconds\n");
	}
	browseButton.setEnabled(false);
	openButton.setEnabled(false);
	flushButton.setEnabled(true);
	closeButton.setEnabled(true);
	this.open = true;
    }

    public void flush() throws IOException {
	this.stream.flush();
    }

    public void close() throws IOException {
	this.open = false;
	this.stream.flush();
	this.stream.close();
	browseButton.setEnabled(true);
	openButton.setEnabled(true);
	flushButton.setEnabled(false);
	closeButton.setEnabled(false);
    }
    
    public void actionPerformed(ActionEvent e) {
        try {
    	    JComponent c = (JComponent) e.getSource();
    	    if(c == fileList) {
                this.file = (String) fileList.getSelectedItem();
                fileList.removeItem(file);
                fileList.addItem(file);
            } else if (c == appendCheck) {
                this.append = appendCheck.isSelected();
            } else if (c == browseButton) {
		/*
                int retval = fileChooser.showOpenDialog(this);
                if (retval == JFileChooser.APPROVE_OPTION) {
                    this.file = fileChooser.getSelectedFile().toString();
		*/
		File tmp = JMeterUtils.promptToOpenFile().getSelectedFile();
		if( tmp != null ) {
		    setFile(tmp.toString());
                }
            } else if (c == openButton) {
                open();
            } else if (c == flushButton) {
                flush();
            } else if (c == closeButton) {
		close();
            } else if (c == verboseCheck) {
                this.verbose = verboseCheck.isSelected();
            } else {
                // do nothing!
            }
        } catch (Exception ex) {
            JOptionPane.showMessageDialog(this, ex, "Error", JOptionPane.ERROR_MESSAGE);
        }
    }

    public synchronized void add(SampleResult sampleResult) {
      long sample = sampleResult.getTime();
        if (open) {
            if (verbose) {
                stream.println(sampleResult.getValue(Sampler.SAMPLE_LABEL) + "  " + sample);
            } else {
                stream.println(sample);
            }
        }
    }

    public synchronized void clear() {
    }

    public String toString() {
        return "Write the samples to a file";
    }

    public JPanel getControlPanel() {
	return this;
    }
}
