/*
 * Copyright (c) 1998-99 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */

package org.apache.jmeter.visualizers;

import java.util.Vector;
import java.awt.*;
import javax.swing.*;

import org.apache.jmeter.samplers.SampleResult;

/**
 * This class implements a statistical analyser that
 * takes samples to process a Spline interpolated curve.
 * Currently, it tries to look mostly like the GraphVisualizer.
 *
 * @author <a href="mailto:norguet@bigfoot.com">Jean-Pierre Norguet</a>
 * @version 0.9.1
 */
public class SplineVisualizer extends JPanel implements Visualizer {

    protected SampleCollection sampleCollection = null; // stores the samples

    protected Spline3 dataCurve = null; // current Spline curve

    protected long sum = 0; // sum of the samples

    protected long average = 0; // average of the samples

    protected long n = 0; // number of collected samples

    private SplineGraph graph = null;

    public final int DEFAULT_NUMBER_OF_NODES = 10;
    public final int DEFAULT_REFRESH_PERIOD = 1;
    protected int numberOfNodes = DEFAULT_NUMBER_OF_NODES;
    protected int refreshPeriod = DEFAULT_REFRESH_PERIOD;

    protected final Color BACKGROUND_COLOR = getBackground();
    protected final Color MINIMUM_COLOR = new Color(0F, 0.5F, 0F);
    protected final Color MAXIMUM_COLOR = new Color(0.9F, 0F, 0F);
    protected final Color AVERAGE_COLOR = new Color(0F, 0F, 0.75F);
    protected final Color INCOMING_COLOR = Color.black;

    protected final boolean SHOW_INCOMING_SAMPLES = true;

    private JLabel minimumLabel = null;
    private JLabel maximumLabel = null;
    private JLabel averageLabel = null;
    private JLabel incomingLabel = null;

    private JLabel minimumNumberLabel = null;
    private JLabel maximumNumberLabel = null;
    private JLabel averageNumberLabel = null;
    private JLabel incomingNumberLabel = null;

    public SplineVisualizer() {
        super();
        this.sampleCollection = new SampleCollection();
        this.setGUI();
    }

    public void setGUI() {
        Color backColor = BACKGROUND_COLOR;
        this.setBackground(backColor);

        graph = new SplineGraph();

        maximumLabel = new JLabel("Maximum");
        maximumLabel.setForeground(MAXIMUM_COLOR);
        maximumLabel.setBackground(backColor);
        averageLabel = new JLabel("Average");
        averageLabel.setForeground(AVERAGE_COLOR);
        averageLabel.setBackground(backColor);
        incomingLabel = new JLabel("Incoming");
        incomingLabel.setForeground(INCOMING_COLOR);
        incomingLabel.setBackground(backColor);
        minimumLabel = new JLabel("Minimum");
        minimumLabel.setForeground(MINIMUM_COLOR);
        minimumLabel.setBackground(backColor);

        maximumNumberLabel = new JLabel("0 ms");
        maximumNumberLabel.setHorizontalAlignment(JLabel.RIGHT);
        maximumNumberLabel.setForeground(MAXIMUM_COLOR);
        maximumNumberLabel.setBackground(backColor);
        averageNumberLabel = new JLabel("0 ms");
        averageNumberLabel.setHorizontalAlignment(JLabel.RIGHT);
        averageNumberLabel.setForeground(AVERAGE_COLOR);
        averageNumberLabel.setBackground(backColor);
        incomingNumberLabel = new JLabel("0 ms");
        incomingNumberLabel.setHorizontalAlignment(JLabel.RIGHT);
        incomingNumberLabel.setForeground(INCOMING_COLOR);
        incomingNumberLabel.setBackground(backColor);
        minimumNumberLabel = new JLabel("0 ms");
        minimumNumberLabel.setHorizontalAlignment(JLabel.RIGHT);
        minimumNumberLabel.setForeground(MINIMUM_COLOR);
        minimumNumberLabel.setBackground(backColor);

        // description Panel
        JPanel labelPanel = new JPanel();
        labelPanel.setLayout(new GridLayout(0, 1));
        labelPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 20));
        labelPanel.setBackground(backColor);
        labelPanel.add(maximumLabel);
        labelPanel.add(averageLabel);
        if (SHOW_INCOMING_SAMPLES) {
            labelPanel.add(incomingLabel);
        }
        labelPanel.add(minimumLabel);
        // number Panel
        JPanel numberPanel = new JPanel();
        numberPanel.setLayout(new GridLayout(0, 1));
        numberPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 20));
        numberPanel.setBackground(backColor);
        numberPanel.add(maximumNumberLabel);
        numberPanel.add(averageNumberLabel);
        if (SHOW_INCOMING_SAMPLES) {
            numberPanel.add(incomingNumberLabel);
        }
        numberPanel.add(minimumNumberLabel);
        // information display Panel
        JPanel infoPanel = new JPanel();
        infoPanel.setLayout(new BorderLayout());
        infoPanel.add(labelPanel, BorderLayout.CENTER);
        infoPanel.add(numberPanel, BorderLayout.EAST);

        setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        setLayout(new BorderLayout());
        add(infoPanel, BorderLayout.WEST);
        add(graph, BorderLayout.CENTER);
        // everyone is free to swing on its side :)
        //add(infoPanel, BorderLayout.EAST);
    }

    public synchronized void add(SampleResult sampleResult) {
      long sample = sampleResult.getTime();
        this.n++;
        this.sum += sample;
        this.average = this.sum / this.n;
        if (SHOW_INCOMING_SAMPLES) {
            this.setIncoming(sample);
        }
        this.sampleCollection.add(sample);
        sampleCollection.add(sample);
        int n = sampleCollection.getNumberOfCollectedSamples();
        if ((n % (numberOfNodes * refreshPeriod)) == 0) {
            float[] floatNode = new float[numberOfNodes];
            long[] longSample = sampleCollection.getSamples();
            // load each node
            int loadFactor = n / numberOfNodes;
            long minNode = Integer.MAX_VALUE;
            long maxNode = Integer.MIN_VALUE;
            for (int i = 0; i < numberOfNodes; i++) {
                for (int j = 0; j < loadFactor; j++) {
                    floatNode[i] += this.sampleCollection.getSample((i * loadFactor) + j);
                }
                floatNode[i] = floatNode[i] / loadFactor;
                if (floatNode[i] < (float)minNode) {
                    minNode = (long)Math.round(floatNode[i]);
                }
                if (floatNode[i] > (float)maxNode) {
                    maxNode = (long)Math.round(floatNode[i]);
                }
            }
            this.setMinimum(minNode);
            this.setMaximum(maxNode);
            this.setAverage(this.average);
            // compute the new Spline curve
            dataCurve = new Spline3(floatNode);
            graph.reinterpolated = true;
        }
        else {
            // do nothing, wait for the next pile to complete
        }
        repaint();
    }

    public synchronized void clear() {
        this.graph.clear();
        this.setMinimum(0);
        this.setMaximum(0);
        this.setAverage(0);
        this.setIncoming(0);
        this.n = 0;
        this.sum = 0;
        this.average = 0;
        this.dataCurve = null;
        this.sampleCollection = new SampleCollection();
        repaint();
    }

    public String toString() {
        return "Show the samples analysis as a Spline curve";
    }


    protected final int NUMBERS_TO_DISPLAY = 4;
    protected final boolean FILL_UP_WITH_ZEROS = false;

    public String formatMeasureToDisplay(long measure) {
        String numberString = String.valueOf(measure);
        if (FILL_UP_WITH_ZEROS) {
            for (int i = numberString.length(); i < NUMBERS_TO_DISPLAY; i++) {
                numberString = "0" + numberString;
            }
        }
        return numberString;
    }

    public void setMinimum(long n) {
        String text = this.formatMeasureToDisplay(n) + " ms";
        this.minimumNumberLabel.setText(text);
    }

    public void setMaximum(long n) {
        String text = this.formatMeasureToDisplay(n) + " ms";
        this.maximumNumberLabel.setText(text);
    }

    public void setAverage(long n) {
        String text = this.formatMeasureToDisplay(n) + " ms";
        this.averageNumberLabel.setText(text);
    }

    public void setIncoming(long n) {
        String text = this.formatMeasureToDisplay(n) + " ms";
        this.incomingNumberLabel.setText(text);
    }


    /**
     *  Component showing a Spline curve.
     */
    public class SplineGraph extends JComponent {

        protected final Color WAITING_COLOR = Color.darkGray;

        protected int lastWidth = -1;
        protected int lastHeight = -1;
        protected int[] plot = null;
        public boolean reinterpolated = false;

        public SplineGraph() {
        }

        /**
         *  Clear the Spline graph and get ready for the next wave.
         */
        public void clear() {
            lastWidth = -1;
            lastHeight = -1;
            plot = null;
            this.repaint();
        }

        public void paintComponent(Graphics g) {
            super.paintComponent(g);

            Dimension dimension = getSize();
            int width = dimension.width;
            int height = dimension.height;

            if (dataCurve == null) {
                g.setColor(getBackground());
                g.fillRect(0, 0, width, height);
                g.setColor(WAITING_COLOR);
                g.drawString( "Waiting for samples",
                              (width - 120) / 2, height - (height - 12) / 2);
                return;
            }

            boolean resized = true;
            if (width == lastWidth && height == lastHeight) {
                // dimension of the SplineGraph is the same
                resized = false;
            }
            else {
                // dimension changed
                resized = true;
                lastWidth = width;
                lastHeight = height;
            }

            if (resized || reinterpolated) {
                // dimension changed or new interpolation made
                // get rescaled plots to draw
                this.plot = dataCurve.getPlots(width, height); // rounds!
                reinterpolated = false;
            }

            int n = plot.length;
            int curY = plot[0];
            for (int i = 1; i < n; i++) {
                g.setColor(Color.black);
                g.drawLine(i - 1, height - curY - 1, i, height - plot[i] - 1);
                curY = plot[i];
            }
        }
    }


    /**
     *  Interface that collections of sample should implement in order
     *  to follow a standard way of coding. :-P
     */
    protected interface SampleCollector {
        public void add(long sample);
        public void clear();
        public long[] getSamples();
        public long getSample(int i);
        public int getNumberOfCollectedSamples();
    }

    /**
     *  Implementation of a Collector based on a Vector.
     */
    protected class SampleCollection implements SampleCollector {

        protected Vector samples = null;

        public SampleCollection() {
            this.samples = new Vector(500, 500);
        }

        public void add(long sample) {
            this.samples.addElement(new Long(sample));
        }

        public void clear() {
            this.samples.removeAllElements();
        }

        public long[] getSamples() {
            int n = samples.size();
            long[] longSample = new long[n];
            for (int i = 0; i < n; i++) {
                longSample[i] = ((Long)samples.elementAt(i)).longValue();
            }
            return longSample;
        }

        public long getSample(int i) {
            Long sample = (Long)this.samples.elementAt(i);
            return sample.longValue();
        }

        public int getNumberOfCollectedSamples() {
            return this.samples.size();
        }

    }

    public JPanel getControlPanel() {
	return this;
    }
}

